/*
 * Copyright (c) 2006-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.oc;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.*;
import org.xml.sax.SAXException;

import org.opencms.jsp.CmsJspBean;
import org.opencms.jsp.Messages;
import org.opencms.file.CmsFile;
import org.opencms.file.CmsResource;
import org.opencms.main.CmsException;

/**
 * Wraps a org.w3c.dom.Document object so it becomes cacheable. 
 *
 */
public class OCCachedXmlDocument implements I_OCCacheElement, I_OCCacheElementWrapper {
	
	/**
	 * Convenience for loading/caching a parsed XML file as a <code>org.w3c.dom.Document</code>.
	 * 
	 * @param cache An initialized OCCache.
	 * @param cms An initialized Cms bean.
	 * @param path The VFS path (without site root).
	 * @return A <code>org.w3c.dom.Document</code> object or <code>null</code>.
	 * @throws CmsException.
	 */
	public static Document cacheXmlDocument(OCCache cache, CmsJspBean cms, String path) throws CmsException {
		OCCachedXmlDocument el = (OCCachedXmlDocument)cache.cacheElement(cms, cms.getRequestContext().addSiteRoot(path), OCCachedXmlDocument.class, path);
		return (el == null ? null : (Document)el.getCacheValue());
	}
	
	/**
	 * Convenience for looking up a cached <code>org.w3c.dom.Document</code> (or load/cache it if it is not in the cache).
	 * 
	 * @param cache An initialized OCCache.
	 * @param cms An initialized Cms bean.
	 * @param path The VFS path (without site root).
	 * @return A <code>org.w3c.dom.Document</code> object or <code>null</code>.
	 * @throws CmsException.
	 */
	public static Document getCachedXmlDocument(OCCache cache, CmsJspBean cms, String path) throws CmsException {
		OCCachedXmlDocument el = (OCCachedXmlDocument)cache.getCachedElement(cms, cms.getRequestContext().addSiteRoot(path), OCCachedXmlDocument.class, path);
		return (el == null ? null : (Document)el.getCacheValue());
	}
	
	// INSTANCE
	
	private String path;
	private boolean online;
	private long expires;
	private long lastmod;
	private Document value;
	
	public OCCachedXmlDocument() {
		super();
	}
	
	/**
	 * Context must be a path to the file (without site root).
	 */
	public boolean init(CmsJspBean cms, boolean online, String key, Object context) throws CmsException {
		this.online = online;
		this.path = (String)context;
		return init(cms);
	}
	
	private boolean init(CmsJspBean cms) throws CmsException {
		CmsResource res = cms.getCmsObject().readResource(path);
		if(res != null) {
			try {
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				DocumentBuilder builder = factory.newDocumentBuilder();
				CmsFile f = CmsFile.upgrade(res,cms.getCmsObject());
				ByteArrayInputStream is = new ByteArrayInputStream(f.getContents());
				setCacheValue(builder.parse(is));
				return true;
			} catch (ParserConfigurationException e) {
				throw new RuntimeException("File = " + path, e);
			} catch (IOException e) {
				throw new RuntimeException("File = " + path, e);
			} catch (SAXException e) {
				throw new CmsException(Messages.get().container("Error parsing xml file: \""+path+"\""), e); // learn to live with it...
			}
		}
		return false;
	}
	
	public boolean refresh(CmsJspBean cms) throws CmsException {
		return init(cms);
	}

	/**
	 * Note that this will try to do the right thing as far as the expire date goes, but the nature of the offline cache's
	 * 'minimum cache period' causes changes to the release/expire dates to go undetected until the next time we check the modified
	 * date (which is changed and subsequently cause a refresh).
	 * 
	 * @param bestBefore
	 * @return
	 */
	public boolean isStale(CmsJspBean cms) throws CmsException {
		if(online) {
			// Online can only become stale because of the expire date (and after publish, but OCCache handles that case)
			return (expires <= System.currentTimeMillis());
		} else {
			CmsResource res = cms.getCmsObject().readResource(path);
			if(res.getDateLastModified() <= lastmod) {
				return false;
			}
			return true;
		}
	}

	public void setCacheValue(Object value) {
		this.value = (Document)value;
	}
	
	public Object getCacheValue() {
		return value;
	}

}
