/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 180 $
 * $LastChangedDate: 2007-06-20 11:22:10 +0200 (Wed, 20 Jun 2007) $
 * 
 */

package com.codedroids.records;

import java.text.ParseException;
import java.util.*;

public class MapRecord extends HashMap implements I_Record {
	
	static final long serialVersionUID =1L;
	
	protected transient RecordFactory factory;
	protected boolean isNew;
	
	public MapRecord() {
		super();
		isNew = true;
	}
	
	public MapRecord(RecordFactory factory) {
		this();
		init(factory);
	}

	// The rest are constructors that instantiates an object from the database
	public MapRecord(RecordFactory factory, String id) throws ParseException, RecordException {
		this();
		init(factory, factory.getIdentField().fromString(id));
	}
	
	public MapRecord(RecordFactory factory, Object id) throws RecordException {
		this();
		init(factory, id);	
	}
	
	public void init(RecordFactory factory) {
		this.factory = factory;
	}
	
	public void init(RecordFactory factory, Object id) throws RecordException {
		this.factory = factory;
		factory.fetchRecord(id, this);
		fetched();
	}
	
	public RecordFactory factory() {
		return factory;
	}
	
	public boolean isNew() {
		//Object pk = valueForKey(factory.getIdentField().getName());
		//return (pk==null);
		return isNew;
	}
	
	public void saved() {
		isNew = false;
	}
	
	public void fetched() {
		isNew = false;
	}
	
	public boolean hasField(String fieldName) {
		return factory.getField(fieldName) != null;
	}
	
	//
	// KVEncoding, primitives are wrapped.
	//
	public void takeValueForField(RecordField field, Object value) {
		if(value==null) {
			super.remove(field.getName());
		} else {
			if(field.fieldClass().isInstance(value))
				super.put(field.getName(), value);
			//else if(field.fieldClass().isPrimitive())
			//	instanceVars.put(field.getName(), value); // some kind of mapping is needed...
			else
				throw new ClassCastException("MapRecord takeValueForKey(): value class was \""+value.getClass().getName()+"\", field \""+field.getName()+"\" expects \"" + field.fieldClass().getName() + "\"");
		}
	}
	
	public Object valueForField(RecordField field) {
		Object o = super.get(field.getName());
		return o;
	}
	
	public void takeValueForKey(String key, Object value) {
		RecordField field ;
		if(key==null || (field = factory.getField(key))==null)
			throw new IllegalArgumentException("MapRecord takeValueForKey(): Unknown field=\""+key+"\"");
		takeValueForField(field, value);
	}
	
	public Object valueForKey(String key) {
		RecordField field ;
		if(key==null || (field = factory.getField(key))==null)
			throw new IllegalArgumentException("MapRecord valueForKey(): Unknown field=\""+key+"\"");
		return valueForField(field);
	}
	
	// Same thing but converts back and forth between string representation and objects
	public void takeStringValueForKey(String key, String value) throws ParseException {
		RecordField field;
		if(key==null || (field = factory.getField(key))==null)
			throw new IllegalArgumentException("MapRecord takeStringValueForKey(): Unknown field=\""+key+"\"");
		takeStringValueForField(field, value);
	}
	
	public void takeStringValueForField(RecordField field, String value) throws ParseException {
		Object o = field.fromString(value);
		takeValueForField(field, o);
	}
	
	public String stringValueForKey(String key) {
		RecordField field;
		if(key==null || (field = factory.getField(key))==null)
			throw new IllegalArgumentException("MapRecord stringValueForKey(): Unknown field=\""+key+"\"");
		return stringValueForField(field);
	}
	
	public String stringValueForField(RecordField field) {
		Object o = valueForField(field);
		return field.toString(o);
	}
	
	// These serves as convenience for handling primitive values, throws if there are class casting problems
	public boolean booleanValueForKey(String key) {
		Boolean o = (Boolean)valueForKey(key);
		return o.booleanValue();
	}
	
	public int intValueForKey(String key) {
		Integer o = (Integer)valueForKey(key);
		return o.intValue();
	}	

	public String toString() {
		StringBuffer buf = new StringBuffer(factory.getInstanceClass()+" (isNew="+isNew()+") - fields:\n");
		Iterator keys = factory.getFields().keySet().iterator();
		while (keys.hasNext()) {
			RecordField field = (RecordField)factory.getFields().get((String)keys.next());
			buf.append(field.getName()+"=["+this.valueForField(field)+"]\n");
		}
		return buf.toString();
	}
	

	/**
	 * Values can be given as strings which then will be parsed which in turn may throw an exception
	 * @param values
	 * @throws ParseException
	 */
	public void takeValuesFromMap(Map values) throws ParseException {
		if(values == null)
			return;
		Iterator i = values.entrySet().iterator();
		while(i.hasNext()) {
			Map.Entry entry = (Map.Entry)i.next();
			RecordField field = factory.getField((String)entry.getKey());
			if(field != null) {
				Object val = entry.getValue();;
				if(val != null && val instanceof String)
					takeStringValueForField(field, (String)val);
			} // else just ignore it...
		}
	}
}
